#!/usr/bin/env python3
import numpy as np
from pyquaternion import Quaternion
import argparse
import matplotlib.pyplot as plt
import sys
from Utils import *

parser = argparse.ArgumentParser(description='Export hinge to vtk.')
parser.add_argument('--f',type=str,default='hinge',help='file name')
parser.add_argument('--t',type=StrToNumpy,default='-1',help='time')
parser.add_argument('--res',type=int,default=10,help='resolution')
parser.add_argument('--apothem',type=float,default=-1,help='apothem')
args = parser.parse_args()

def GetBeads(fname,time,a=-1):
	my_file=''
	if fname[-4:] == ".txt":
		my_file = fname
	else:
		tExtension = ''
		if time>-1:
			tExtension = '_t{t:05d}'.format(t=time)
		my_file = fname + tExtension + '.txt'
	if exists(my_file):
		rad = np.loadtxt(my_file,usecols=(1,),unpack=True)
		pos = np.loadtxt(my_file,usecols=(4,5,6),unpack=True).transpose()
		if a>0:
			u = np.array([0,0,1])
			v = pos[-1] - pos[0]
			v = v/np.linalg.norm(v)
			w = np.cross(u,v)
			posa = np.array([posi + a*w/2 for posi in pos])
			posb = np.array([posi - a*w/2 for posi in pos])
			pos = np.concatenate((posa,posb))
			rad = np.concatenate((rad,rad))
		return pos,rad
	else:
		print( '  Error: ' + my_file + ' does not exist')
		return np.array([[-1,-1,-1]]),np.array([-1])

def VTKScript(positions,radii,res,fname,time):
	original_stdout = sys.stdout
	tExtension = ''
	out = ''
	if fname[-4:] == ".txt":
		out = fname[:-4]
	else:
		if time>-1:
			tExtension = '_t{:05d}'.format(time)
		out = fname + tExtension
	with open(out + '.py', 'w') as f:
		sys.stdout = f
		print("print(\"  Write {} in vtk format\")".format(out))
		print("from paraview.simple import *")
		for b,(positionb,radb) in enumerate(zip(positions,radii)):
			bExtension='_b{b:02}'.format(b=b)
			vtkFileName = fname + '{b_ext}{t_ext}.vtk'.format(b_ext=bExtension,t_ext=tExtension)
			print("sphere{b} = Sphere()".format(b=b))
			print("sphere{b}.Center = [{xc}, {yc}, {zc}]".format(b=b,xc=positionb[0],yc=positionb[1],zc=positionb[2]))
			print("sphere{b}.Radius = {rad}".format(b=b,rad=radb))
			print("sphere{b}.PhiResolution = {res}".format(b=b,res=res))
			print("sphere{b}.ThetaResolution = {res}".format(b=b,res=res))
			print("SaveData('{out}', proxy=sphere{b})".format(b=b,out=vtkFileName))
		sys.stdout = original_stdout
	print("  Write",out + '.py')
	return(0)

print("  Exporting hinge to vtk (.py)")
for t in args.t:
	positions,radii = GetBeads(args.f,t,args.apothem)
	if np.all(radii != -1):
		VTKScript(positions,radii,args.res,args.f,t)
