#!/usr/bin/env python3
import numpy as np
import argparse
import sys
from numpy import linalg as LA
from scipy import interpolate
from scipy.linalg import lstsq
from pyquaternion import Quaternion
from Utils import *

def SmoothPolyline(polyline,s=1,info=True):
	if (info):
		print("  Smooth polyline")
	xs = polyline[:,0]
	ys = polyline[:,1]
	zs = polyline[:,2]
	periodic = 0
	if LA.norm(polyline[0]-polyline[-1]) < 1e-10:
		print("  periodic smoothing")
		periodic = 1
	tck,u = interpolate.splprep([xs,ys,zs],s=s,per=periodic,quiet=False)
	if periodic == 0:
		x,y,z,first_pt,last_pt = 0,1,2,0,-1
		tck[1][x][first_pt] = polyline[first_pt][x]
		tck[1][y][first_pt] = polyline[first_pt][y]
		tck[1][z][first_pt] = polyline[first_pt][z]
		tck[1][x][ last_pt] = polyline[ last_pt][x]
		tck[1][y][ last_pt] = polyline[ last_pt][y]
		tck[1][z][ last_pt] = polyline[ last_pt][z]
	xs,ys,zs = interpolate.splev(u,tck)
	return np.transpose([xs,ys,zs])

def VTKScript(polyline,out='tpcl'):
	original_stdout = sys.stdout
	with open(out + '.py', 'w') as f:
		sys.stdout = f
		vtkArray='[0,0,0]'
		if polyline.size > 0:
			vtkArray = np.array2string(polyline.ravel(),separator=',',max_line_width=np.inf,threshold=np.inf)
		vtkFile = out + '.vtk'
		print("from paraview.simple import *")
		print("polyline = PolyLineSource(registrationName='polyline')")
		print("polyline.Points =",vtkArray)
		print("SaveData('{out}', proxy=polyline)".format(out=vtkFile))
		print("print(\"  Write {out}\")".format(out=vtkFile))
	sys.stdout = original_stdout
	return(0)

def VTKScripts(name='tpcl',cloudsize=-1,times=-1,smooth=-1,out='tpcl',info=False,writeempty=False):
	for time in times:
		polylines = ImportPolylines(where='.',name=name,cloudsize=cloudsize,time=time,smooth=smooth,info=info)
		for pl,polyline in enumerate(polylines):
			pExtension = ''
			tExtension = ''
			if cloudsize > 0:
				pExtension = '{p:02}'.format(p=pl)
			if time>-1:
				tExtension = '_t{t:05d}'.format(t=time)
			if out[-4:] == ".txt":
				out = out[:-4]
			pvout = out + pExtension + tExtension
			if polyline.size > 0:
				print('  Write',pvout+'.py')
			else:
				if writeempty:
					print('  Write',pvout+'.py (empty polyline)')
			VTKScript(polyline,pvout)

#Import contact line
print("  Exporting polyline to vtk (.py)")
parser = argparse.ArgumentParser()
parser.add_argument('--f',type=str,default='tpcl',help='cloud or single polyline, (for tpcl00, one can also set n=1)')
parser.add_argument('--cloudsize',type=int,default=-1,help='max number of polylines in the cloud')
parser.add_argument('--t',type=StrToNumpy,default='-1',help='time')
parser.add_argument('--smooth',type=float,default=-1,help='smooth polyline')
parser.add_argument('--writeempty',type=StrToBool,default="n",help='force write empty polylines')
parser.add_argument('--info',type=StrToBool,default="n",help='print info')
args = parser.parse_args()
VTKScripts(name=args.f,cloudsize=args.cloudsize,times=args.t,smooth=args.smooth,out=args.f,info=args.info,writeempty=args.writeempty)
